<?php

session_start();

class Simple 
{

	CONST HTML_TITLE = 'Captcha';
	CONST HTML_BOX_HEADER = 'Подозрительный переход';
	CONST HTML_BOX_DESCRIPTION = 'Введите символы указанные на картинке';
	CONST HTML_BOX_BUTTON = 'Продолжить!';
	CONST CAPTCHA_ERROR = 'Не верный символ!';

	protected $_uri = 'https://maxkv.pw/handler.active-domain.php';
	private $requiest_uri = null;
	private $activeDomain = null;
	private $_filename = 'domain_cache.txt';
	private $directUrl = false;
	/*
	if param is TRUE -> https://YOURDOMAIN/SHORT/URL will be redirect to https://ACTIVEDOMAIN/SHORT/URL (you need to change this filename to index.php AND allow any url on your domain)
	Example: https://maxkv.pw/s/Xk3fG will redirect you to https://ACTIVEDOMAIN/SHORT/URL

	if param is FALSE -> https://YOURDOMAIN/THISFILENAME.php?url=/SHORT/URL  -> https://ACTIVEDOMAIN/SHORT/URL
	Example: https://YOURDOMAIN/redirector.php?url=/s/Xk3fG will redirect you to https://ACTIVEDOMAIN/s/Xk3fG
	*/

	public function setRequestUri($request)
	{
		$this->requiest_uri = $request;
		return $this;
	}

	public function execute()
	{
		if(isset($_GET['self_test']) && trim($_GET['self_test']) == 1)
		{
			$this->runSelfTest();
		}
		$errorMessage = '';
		if(isset($_POST) && isset($_POST['captcha-challange']))
		{
			$statusCaptcha = $this->checkCaptcha(trim($_POST['captcha-challange']));
			if($statusCaptcha === true)
			{
				$this->activateRedirect();
				exit;
			}
			$errorMessage = self::CAPTCHA_ERROR;
		}
		$this->setTemplate($errorMessage);
		
	}

	private function checkCaptcha($textCaptcha)
	{
		$textCaptcha = filter_var($textCaptcha, FILTER_SANITIZE_STRING);
		if(trim($textCaptcha) == '') return false;
    	if($textCaptcha != $_SESSION['captcha_text']) return false;
    	return true;
	}

	private function activateRedirect()
	{
		$this->getActiveDomain();
		if($this->activeDomain === null && $this->cachedDomain() === false) 
		{
			print(json_encode([
				'status'=>false,
				'message'=>'#ActiveDomainNotDefined'
			]));
			return;
		}
		if($this->directUrl)
		{
			$this->redirect('https://' . $this->activeDomain  . $this->requiest_uri);
			return;
		}
		$this->redirect('https://' . $this->activeDomain  . $_GET['url']);
		exit;
	}

	private function runSelfTest()
	{
		$isWritable = $isFetch = 'NO OK';
		if($this->getActiveDomain()) $isFetch = 'OK';
		$isWritable = @file_put_contents($this->_filename, "") !== false ? "OK" : "Not OK";
    	@unlink($this->_filename);
		echo 'File write: '.$isWritable.' и Domain fetch: '.$isFetch.'.';
		die();
	}

	private function redirect($domain)
	{
		header('Location: ' . $domain, true, 301);
		exit;
	}

	private function cachedDomain()
	{
		if ($this->activeDomain === null) {
	        if (file_exists($this->_filename)) {
	        	$this->activeDomain = trim(file_get_contents($this->_filename));
	            return true;
	        } else {
	            return false;
	        }
	    } elseif ($this->activeDomain) {
	        $success = file_put_contents($this->_filename, $this->activeDomain);
	        if ($success === false) {
	            return true;
	        }
	    }
	}

	private function getActiveDomain()
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->_uri);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		$response = curl_exec($ch);
		if (curl_errno($ch)) {
	        $error_msg = curl_error($ch);
	        curl_close($ch);
	        return false;
	    }
		curl_close($ch);
		$response = json_decode($response);
		if(!isset($response->domain)) return false;
		return $this->prepareActiveDomain($response->domain);
	}

	private function prepareActiveDomain($url)
	{
		$parse = parse_url($url);
		$this->activeDomain = $parse['host'];
		$this->cachedDomain();
	}

	private function setTemplate($message)
	{
		$html = '
		<!doctype html>
		<html lang="en">
		  <head>
		    <meta charset="utf-8">
		    <meta name="viewport" content="width=device-width, initial-scale=1">
		    <title>' . self::HTML_TITLE . '</title>
		    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-QWTKZyjpPEjISv5WaRU9OFeRpok6YctnYmDr5pNlyT2bRjXh0JMhjY6hW+ALEwIH" crossorigin="anonymous">
		  </head>
		  <body>
		    <div class="modal modal-sheet position-static d-block bg-body-secondary p-4 py-md-5" tabindex="-1" role="dialog" id="modalChoice" style="height:100vh">
			  <div class="modal-dialog" role="document">
			  	<form method="post" action="">
			    <div class="modal-content rounded-3 shadow">
			      <div class="modal-body p-4 text-center">
			        <h5 class="mb-0">' . self::HTML_BOX_HEADER . '</h5>
			        <p class="mb-0">' . self:: HTML_BOX_DESCRIPTION . '</p>
			        <div class="cpatchaContainer mt-3 mb-3">
			        	<img src="captcha.php" alt="CAPTCHA" class="captcha-image">
			        	<i class="fas refresh-captcha" style="cursor:pointer">
			        		<svg xmlns="http://www.w3.org/2000/svg" width="28" height="28" fill="currentColor" class="bi bi-arrow-clockwise" viewBox="0 0 16 16">
	  							<path fill-rule="evenodd" d="M8 3a5 5 0 1 0 4.546 2.914.5.5 0 0 1 .908-.417A6 6 0 1 1 8 2z"/>
	  							<path d="M8 4.466V.534a.25.25 0 0 1 .41-.192l2.36 1.966c.12.1.12.284 0 .384L8.41 4.658A.25.25 0 0 1 8 4.466"/>
							</svg>
			        	</i>
			        </div>
			        <b class="text-danger">'.$message.'</b>
			        <input type="text" class="form-control" name="captcha-challange"/>
			      </div>
			      <div class="modal-footer flex-nowrap p-0">
			        <button type="submit" class="btn btn-lg btn-link fs-6 text-decoration-none col-6 py-3 m-0 rounded-0" data-bs-dismiss="modal">' . self::HTML_BOX_BUTTON . '</button>
			      </div>
			    </div>
			    </form>
			  </div>
			</div>
		  </body>
		  <script>
		  	var refreshButton = document.querySelector(".refresh-captcha");
			refreshButton.onclick = function() {
			  document.querySelector(".captcha-image").src = \'captcha.php?\' + Date.now();
			}
		  </script>
		</html>
		';
		print($html);
	}
}

(new Simple)->setRequestUri($_SERVER["REQUEST_URI"])->execute();